(* Title:  All tests on isac (some outcommented since Isabelle2002-->2009-2)
   Author: Walther Neuper 101001
   (c) copyright due to lincense terms.

   Isac's tests are organised parallel to sources: 
     "~~/test/Tools/isac" has same directory structure as "~~/src/Tools/isac"
   plus
     ~~/test/Tools/isac/ADDTESTS
     ~~/test/Tools/isac/Minisubpbl: the Lucas-Interpreter's core functionality

$ cd /usr/local/isabisac/
$ ./bin/isabelle jedit -l Isac test/Tools/isac/Test_Isac.thy &
*)

(* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! *)
(* !!!!! say "OK" to the popup asking for theories to be loaded and !!!!!!!!!!!!! *)
(* !!!!! watch the <Theories> window for errors in the "imports" below !!!!!!!!!! *)
(* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! *)

theory Test_Isac imports Build_Thydata
  "ADDTESTS/accumulate-val/Thy_All"
  "ADDTESTS/Ctxt"
  "ADDTESTS/test-depend/Build_Test"
  "ADDTESTS/All_Ctxt"
  "ADDTESTS/course/phst11/T1_Basics"
  "ADDTESTS/course/phst11/T2_Rewriting"
  "ADDTESTS/course/phst11/T3_MathEngine"
  "ADDTESTS/file-depend/BuildC_Test"
  "ADDTESTS/session-get_theory/Foo"
(*"ADDTESTS/course/SignalProcess/Build_Inverse_Z_Transform"
   ADDTESTS/------------------------------------------- see end of tests *)
  "~~/test/Pure/Isar/Test_Parsers"
(*"~~/test/Pure/Isar/Pure/Isar/Struct_Deriv" lost at update 2009-2-->2011*)
  "~~/test/Pure/Isar/Test_Parse_Term"
  "~~/test/HOL/Library/Test_Polynomial"
  "~~/test/Tools/isac/Interpret/ptyps"       (* setup for ptyps.sml    *)
  "~~/test/Tools/isac/ProgLang/calculate"    (* setup for calculate.sml*)
  "~~/test/Tools/isac/ProgLang/scrtools"     (* setup for scrtools.sml *)
  "~~/test/Tools/isac/Knowledge/integrate"   (* setup for integrate.sml*)

  "~~/src/Tools/isac/Knowledge/GCD_Poly_OLD" (*not imported by Isac.thy*)
  "~~/src/Tools/isac/Knowledge/GCD_Poly_FP"  (*not imported by Isac.thy*)

begin

ML {*
  KEStore_Elems.set_ref_thy @{theory};
  fun autoCalculate' cI auto = autoCalculate cI auto (*|> Future.join*);
*}

section {* trials with Isabelle's functions *}
  ML {*"%%%%%%%%%%%%%%%%% start Isabelle %%%%%%%%%%%%%%%%%%%%%%%";*}
  ML_file "~~/test/Pure/General/alist.ML"
  ML_file "~~/test/Pure/General/basics.ML"
  ML_file "~~/test/Pure/General/scan.ML"
  ML_file "~~/test/Pure/PIDE/xml.ML"
  ML {*"%%%%%%%%%%%%%%%%% end Isabelle %%%%%%%%%%%%%%%%%%%%%%%%%";*}

section {* test ML Code of isac *}
  ML {*"%%%%%%%%%%%%%%%%% start ProgLang.thy %%%%%%%%%%%%%%%%%%%";*}
  ML_file          "library.sml"
  ML_file          "calcelems.sml"
  ML_file          "kestore.sml"
  ML_file "ProgLang/termC.sml"
  ML_file "ProgLang/calculate.sml"      (* requires setup from calculate.thy *)
  ML_file "ProgLang/rewrite.sml"
  ML_file "ProgLang/listC.sml"
  ML_file "ProgLang/scrtools.sml"
  ML_file "ProgLang/tools.sml"
  ML {*"%%%%%%%%%%%%%%%%% end ProgLang.thy %%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start Minisubpbl %%%%%%%%%%%%%%%%%%%%%";*}
  ML_file "Minisubpbl/000-comments.sml"
  ML_file "Minisubpbl/100-init-rootpbl.sml"
  ML_file "Minisubpbl/150-add-given.sml"
  ML_file "Minisubpbl/200-start-method.sml"
  ML_file "Minisubpbl/300-init-subpbl.sml"
  ML_file "Minisubpbl/400-start-meth-subpbl.sml"
  ML_file "Minisubpbl/490-nxt-Check_Postcond.sml"
  ML_file "Minisubpbl/500-met-sub-to-root.sml"
  ML_file "Minisubpbl/530-error-Check_Elementwise.sml"
  ML_file "Minisubpbl/600-postcond.sml"
  ML {*"%%%%%%%%%%%%%%%%% end Minisubpbl.thy %%%%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start Interpret.thy %%%%%%%%%%%%%%%%%%";*}
  ML_file "Interpret/mstools.sml"
  ML_file "Interpret/ctree.sml"         (*!...!see(25)*)
  ML_file "Interpret/ptyps.sml"         (* requires setup from ptyps.thy *)
  ML {* (*check_unsynchronized_ref (); ==== trick on error: CUT AND PASTE THIS LINE =========*) *}
(*TRICK DOESN'T WORK: get_pbt not found: ["IsacKnowledge","Diff","Theorems","diff_sin_chain"]*)
  ML_file "Interpret/generate.sml"
(*WITHOUT inhibit exn WN1130621 Isabelle2012-->13 !thehier! THIS ERROR OCCURS:
  ... SAME ERROR HERE ON ISABELLE2012 AS IN ISAC ON ISABELLE2011*)
  ML_file "Interpret/calchead.sml"
  ML_file "Interpret/appl.sml"          (*complete "WEGEN INTERMED TESTCODE"                          *)
  ML_file "Interpret/rewtools.sml"
  ML_file "Interpret/script.sml"
  ML_file "Interpret/solve.sml"
  ML_file "Interpret/inform.sml"
(*WITHOUT inhibit exn WN1130621 Isabelle2012-->13 !thehier! THIS ERROR OCCURS:
  ... SAME ERROR HERE ON ISABELLE2012 AS IN ISAC ON ISABELLE2011*)
  ML_file "Interpret/mathengine.sml"    (*!part. WN130804: +check Interpret/me.sml*)
  ML {*"%%%%%%%%%%%%%%%%% end Interpret.thy %%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start xmlsrc.thy %%%%%%%%%%%%%%%%%%%%%";*}
  ML_file "xmlsrc/mathml.sml"           (*part.*)
  ML_file "xmlsrc/datatypes.sml"        (*TODO/part.*)
  ML_file "xmlsrc/pbl-met-hierarchy.sml"(*TODO after 2009-2/part.*)
  ML_file "xmlsrc/thy-hierarchy.sml"
  ML_file "xmlsrc/interface-xml.sml"     (*TODO after 2009-2*)
  ML {*"%%%%%%%%%%%%%%%%% end xmlsrc.thy %%%%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start Frontend.thy %%%%%%%%%%%%%%%%%%%";*}
  ML_file "Frontend/messages.sml"
  ML_file "Frontend/states.sml"
  ML_file "Frontend/interface.sml"
  ML_file "Frontend/use-cases.sml"
(*WITHOUT inhibit exn WN1130621 Isabelle2012-->13 !thehier! THIS ERROR OCCURS:
  ... SAME ERROR HERE ON ISABELLE2012 AS IN ISAC ON ISABELLE2011*)
  ML_file          "print_exn_G.sml"
  ML {*"%%%%%%%%%%%%%%%%% end Frontend.thy %%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start Knowledge %%%%%%%%%%%%%%%%%%%%%%";*}
  ML_file "Knowledge/delete.sml"
  ML_file "Knowledge/descript.sml"
  ML_file "Knowledge/atools.sml"
  ML_file "Knowledge/simplify.sml"
  ML_file "Knowledge/poly.sml"
  ML_file "Knowledge/gcd_poly_ml.sml"
  ML_file "Knowledge/gcd_poly_winkler.sml" (*must be after gcd_poly_ml.sml: redefines functions*)
  ML_file "Knowledge/rational.sml"
  ML_file "Knowledge/equation.sml"
  ML_file "Knowledge/root.sml"
  ML_file "Knowledge/lineq.sml"
(*ML_file "Knowledge/rooteq.sml"    some complicated equations not recovered from 2002 *)
  ML_file "Knowledge/rateq.sml"   (*some complicated equations not recovered from 2002 *)
  ML_file "Knowledge/rootrat.sml"
  ML_file "Knowledge/rootrateq.sml"(*ome complicated equations not recovered from 2002 *)
  ML_file "Knowledge/partial_fractions.sml"
  ML_file "Knowledge/polyeq.sml"
(*ML_file "Knowledge/rlang.sml"     much to clean up, similar tests in other files     *)
  ML_file "Knowledge/calculus.sml"
  ML_file "Knowledge/trig.sml"
(*ML_file "Knowledge/logexp.sml"    not included as stuff for presentation of authoring*) 
  ML_file "Knowledge/diff.sml"
  ML_file "Knowledge/integrate.sml"
  ML_file "Knowledge/eqsystem.sml"
  ML_file "Knowledge/test.sml"
  ML_file "Knowledge/polyminus.sml"
  ML_file "Knowledge/vect.sml"
  ML_file "Knowledge/diffapp.sml"        (* postponed to dev. specification | TP-prog. *)
  ML_file "Knowledge/biegelinie.sml"
  ML_file "Knowledge/algein.sml"
  ML_file "Knowledge/diophanteq.sml"
  ML_file "Knowledge/Inverse_Z_Transform/inverse_z_transform.sml"
  ML_file "Knowledge/isac.sml"
  ML_file "Knowledge/build_thydata.sml"
  ML {*"%%%%%%%%%%%%%%%%% end Knowledge %%%%%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% start ADDTESTS %%%%%%%%%%%%%%%%%%%%%%%";*}
  ML_file "~~/test/Tools/isac/ADDTESTS/libisabelle/mini-test.sml"
  ML_file "~~/test/Tools/isac/ADDTESTS/libisabelle/protocol.sml"
  ML {*"%%%%%%%%%%%%%%%%% end ADDTESTS %%%%%%%%%%%%%%%%%%%%%%%%%";*}

  ML {*"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%% end Test_Isac %%%%%%%%%%%%%%%%%%%%%%%%";*}
  ML {*"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";*}

section {* history of tests *}
text {*
  Systematic regression tests have been introduced to isac development in 2003.
  Sanity of the regression tests suffers from updates following Isabelle development,
  which mostly exceeded the resources available in isac's development.

  The survey below shall support to efficiently use the tests for isac 
  on different Isabelle versions. Conclusion in most cases will be: 

  !!! Use most recent tests or go back to the old notebook
      with isac on Isabelle2002. !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*}

subsection {* isac on Isabelle2014 *}
subsubsection {* Summary of development *}
text {*
  migration from "isabelle tty" --> libisabelle
*}

subsection {* isac on Isabelle2013-2 *}
subsubsection {* Summary of development *}
text {*
  reactivated context_thy
*}
subsubsection {* State of tests *}
text {*
  TODO
*}
subsubsection {* Changesets of begin and end *}
text {*
  TODO
  :
  : isac on Isablle2013-2
  :
  Changeset: 55318 (03826ceb24da) merged
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-12-12 14:27:37 +0100 (7 minutes)
*}

subsection {* isac on Isabelle2013-1 *}
subsubsection {* Summary of development *}
text {*
  Isabelle2013-1 was replaced within a few weeks due to problems with the document model;
  no significant development steps for ISAC.
*}
subsubsection {* State of tests *}
text {*
  See points in subsection "isac on Isabelle2011", "State of tests".
*}
subsubsection {* Changesets of begin and end *}
text {*
  Changeset: 55283 (d6e9a34e7142) notes for resuming work on Polynomial.thy
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-12-03 18:13:31 +0100 (8 days)
  :
  : isac on Isablle2013-1
  :
  Changeset: 55279 (130688f277ba) Isabelle2013 --> 2013-1: Test_Isac perfect
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-11-21 18:12:17 +0100 (2 weeks)

*}

subsection {* isac on Isabelle2013 *}
subsubsection {* Summary of development *}
text {*
  # Oct.13: replaced "axioms" by "axiomatization"
  # Oct.13: Mathias Lehnfeld started removing Unsynchornized.ref
  # Sep.13: integrated gcd_poly (functional, without Unsychronized.ref) into
    simplification of multivariate rationals (without improving the rulesets involved).
*}
subsubsection {* Run tests *}
text {*
  Is standard now; this subsection will be discontinued under Isabelle2013-1
*}
subsubsection {* State of tests *}
text {*
  See points in subsection "isac on Isabelle2011", "State of tests".
  # re-activated listC.sml
*}
subsubsection {* Changesets of begin and end *}
text {*
  changeset 52174:8b055b17bd84 --- removed all code concerned with "castab = Unsynchronized.ref"
  User: Mathias Lehnfeld <s1210629013@students.fh-hagenberg.at>
  Date: Tue Nov 19 22:23:30 2013 +0000
  :
  : isac on Isablle2013 
  :
  Changeset: 52061 (4ecea2fcdc2c) --- Build_Isac.thy runs on Isabelle2013
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-07-15 08:28:50 +0200 (4 weeks)
*}

subsection {* isac on Isabelle2012 *}
subsubsection {* Summary of development *}
text {*
  isac on Isabelle2012 is considered just a transitional stage
  within the update from Isabelle2011 to Isabelle2013; thus no further development of isac;
  For considerations on the transition see 
  ~~/src/Tools/isac/Knowledge/Build_Thydata/thy, section "updating isac..".
*}
subsubsection {* Run tests *}
text {*
$ cd /usr/local/isabisac12/
$ ./bin/isabelle jedit -l HOL src/Tools/isac/Build_Isac.thy
$ ./bin/isabelle jedit -l Isac test/Tools/isac/Test_Isac.thy
*}
subsubsection {* State of tests *}
text {*
  At least the tests from isac on Isabelle2011 run again.
  However, Test_Isac.thy shows erratic behaviour; no errors are obtained when scrolling 
  in parallel with evaluation.

  Counting "error in kernel" for Frontend/interface.sml (the tests considered most significant)
  yields 69 hits, some of which were already present before Isabelle2002-->2009-2
  (i.e. on the old notebook from 2002).

  Now many tests with (*...=== inhibit exn ...*) give a reason or at least the origin:
  # === inhibit exn WN1130621 Isabelle2012-->13 !thehier! === ...see Build_Thydata.thy
  # === inhibit exn AK110726 === ...reliable work by Alexander Kargl, most likely go back to 2002
  # === inhibit exn WN1130701 broken at Isabelle2002 --> 2009-2 === , most likely go back to 2002
  Reasons for outcommented tests are also found in Test_Isac.thy near the respective file.sml.

  Some tests have been re-activated (e.g. error patterns, fill patterns).
*}
subsubsection {* Changesets of begin and end *}
text {*  
  Changeset: 52051 (35751d90365e) end of improving tests for isac on Isabelle2012
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-07-11 16:58:31 +0200 (4 weeks)
  :
  : isac on Isablle2012 
  :
  Changeset: 48757 (74eb3dfc33cc) updated src from Isabelle2011 to Isabelle2012
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 18:35:13 +0200 (8 months)
  ------------------------------------------------------------------------------
  Changeset: 48756 (7443906996a8) merged
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 18:15:49 +0200 (8 months)
*}

subsection {* isac on Isabelle2011 *}
subsubsection {* Summary of development *}
text {*
  isac's mathematics engine has been extended by two developments:
  (1) Isabelle's contexts were introduced by Mathias Lehnfeld
  (2) Z_Transform was introduced by Jan Rocnik, which revealed
    further errors introduced by (1).
  (3) "error patterns" were introduced by Gabriella Daroczy
  Regressions tests have been added for all of these.
*}
subsubsection {* Run tests *}
text {*
  $ cd /usr/local/isabisac11/
  $ ./bin/isabelle jedit -l HOL src/Tools/isac/Build_Isac.thy
  $ ./bin/isabelle jedit -l Isac test/Tools/isac/Test_Isac.thy
*}
subsubsection {* State of tests *}
text {* 
  Systematic efforts outcommented less significant tests by (*...=== inhibit exn ...*) 
  and sometimes give reasons for failing tests.
  (*...=== inhibit exn AK...*) was done by Alexander Kargl; this is reliable
  work, some of which couldn't be revised (and renamed) by WN and thus survived some time.

  The most signification tests (in particular Frontend/interface.sml) run,
  however, many "error in kernel" are not caught by an exception.
  ------------------------------------------------------------------------------
  After the changeset below Test_Isac worked with check_unsynchronized_ref ():
  ------------------------------------------------------------------------------
  Changeset: 42457 (ca691a84b81a) PROVISIONALLY MADE TESTS RUN with Unsynchronized.ref
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-08-06 10:38:11 +0200 (11 months)


  The list below records TODOs while producing an ISAC kernel for 
  gdaroczy and jrocnik, wich could NOT be done before all tests are RUNNING
  (so to be resumed with Isabelle2013-1):
  ############## WNxxxxxx.TODO can be found in sources ##############
  --------------------------------------------------------------------------------
  WN111013.TODO: lots of cleanup/removal in test/../Test.thy
  --------------------------------------------------------------------------------
  WN111013.TODO: remove concept around "fun init_form", lots of troubles with 
  this special case (see) --- why not nxt = Model_Problem here ? ---
  --------------------------------------------------------------------------------
  WN111014.TODO calculate_Poly < calculate_Rational < calculate_RootRat, see test/
  ... FIRST redesign 
  # simplify_* , *_simp_* 
  # norm_* 
  # calc_* , calculate_*  ... require iteration over all rls ...
  ... see --- val rls = calculate_RootRat > calculate_Rational --- CONTINUE !
  --------------------------------------------------------------------------------
  WN111014.TODO fun prep_rls | !!!use this function in ruleset' := !!!
  --------------------------------------------------------------------------------
  WN120314 changeset a393bb9f5e9f drops root equations.
  see test/Tools/isac/Knowledge/rootrateq.sml 
  --------------------------------------------------------------------------------
  WN120317.TODO changeset 977788dfed26 dropped rateq:
  # test --- repair NO asms from rls RatEq_eliminate --- shows error from 2002
  # test --- solve (1/x = 5, x) by me --- and --- x / (x ^ 2 - 6 * x + 9) - ...:    
    investigation Check_elementwise stopped due to too much effort finding out,
    why Check_elementwise worked in 2002 in spite of the error.
  --------------------------------------------------------------------------------
  WN120317.TODO postponed test/../ratinal,ratinal2.sml to joint work with dmeindl 
  --------------------------------------------------------------------------------
  WN120317.TODO found by test --- interSteps for Schalk 299a --- that 
    NO test with 'interSteps' is checked properly (with exn on changed behaviour)
  --------------------------------------------------------------------------------
  WN120317.TODO test --- Matthias Goldgruber 2003 rewrite orders --- has
    a newly outcommented test where rewrite_set_ make_polynomial --> NONE
  --------------------------------------------------------------------------------
  WN120320.TODO check-improve rlsthmsNOTisac:
  DONE make test --- old compute rlsthmsNOTisac by eq_thmI'
  DONE compare rlsthmsNOTisac in thms-survey-Isa02-Isa09-2.sml .. Isac.thy 
  FOUND 120321: Theory.axioms_of doesnt find LENGTH_CONS etc, thus are in Isab
  # mark twice thms (in isac + (later) in Isabelle) in Isac.thy
  --------------------------------------------------------------------------------
  WN120320.TODO rlsthmsNOTisac: replace twice thms ^
  --------------------------------------------------------------------------------
  WN120320.TODO rlsthmsNOTisac: reconsider design of sym_* thms, see test
  --- OLD compute rlsthmsNOTisac by eq_thmID ---: some are in isab, some in isac.
  --------------------------------------------------------------------------------
  WN120321.TODO rearrange theories:
    Knowledge
      :
      Atools.thy
      ///Descript.thy --> ProgLang
      Delete.thy   <--- first_Knowledge_thy (*mv to Atools.thy*)
    ProgLang: restructure Build_Isac.thy such that no xmlsrc.thy, ProgLang.thy
          Interpret.thy are generated (simplifies xml structure for theories)
      Script.thy
      Tools.thy
      ListC.thy    <--- first_Proglang_thy
  --------------------------------------------------------------------------------
  WN120321.TODO reanimate test/../simplify.sml: CAS-command Simplify
      EXN "simplify.sml: diff.behav. CAScmd: Simplify (2*a + 3*a)"
  broken during work on thy-hierarchy
  --------------------------------------------------------------------------------
  WN120321.TODO LAST in IsacScripts + in IsacKnowledge, see
  test --- the_hier (get_thes ()) (collect_thydata ())---
  --------------------------------------------------------------------------------
  WN120405a.TODO src/../pbl-met-hierarchy.sml: fun pbl2term--> Isac' instead Isac_
  !!add mutual crossreferences to ?fun headline??? where the same has to be done:
  !!OR BETTER: use only 2 functions for adding/removing "'" to/from thy!!
  --------------------------------------------------------------------------------
  WN120409.TODO replace "op mem" (2002) with member (2011) ... 
  ... an exercise interesting for beginners !
  --------------------------------------------------------------------------------
  WN120411 scanning html representation of newly generated knowledge:
  * thy:
  ** Theorems: only "Proof of the theorem" (correct!)
               and "(c) isac-team (math-autor)"
  ** Rulesets: only "Identifier:///"
               and "(c) isac-team (math-autor)"
  ** IsacKnowledge: link to dependency graph (which needs to be created first)
  ** IsacScripts --> ProgramLanguage
  *** Tools: Theorems: arity_type_cp, arity_type_nam, ... wegnehmen
  
  * pbl: OK !?!
  * met: OK !?!
  * exp: 
  ** Z-Transform is missing !!!
  ** type-constraints !!!
  --------------------------------------------------------------------------------
  WN120417: merging xmldata revealed:
  ..............NEWLY generated:........................................
  <THEOREMDATA>
    <GUH> thy_isab_Fun-thm-o_apply </GUH>
    <STRINGLIST>
      <STRING> Isabelle </STRING>
      <STRING> Fun </STRING>
      <STRING> Theorems </STRING>
      <STRING> o_apply </STRING>
    </STRINGLIST>
      <MATHML>
        <ISA> (?f o ?g) ?x = ?f (?g ?x) </ISA>
      </MATHML>  <PROOF>
      <EXTREF>
        <TEXT> Proof of the theorem </TEXT>
        <URL> http://www.ist.tugraz.at/projects/isac/www/kbase/thy/browser_info/HOL/HOL-Real/Isac/Fun.html </URL>
      </EXTREF>
    </PROOF>
    <EXPLANATIONS> </EXPLANATIONS>
    <MATHAUTHORS>
      <STRING> Isabelle team, TU Munich </STRING>
    </MATHAUTHORS>
    <COURSEDESIGNS>
    </COURSEDESIGNS>
  </THEOREMDATA>
  ..............OLD FORMAT:.............................................
  <THEOREMDATA>
    <GUH> thy_isab_Fun-thm-o_apply </GUH>
    <STRINGLIST>
      <STRING> Isabelle </STRING>
      <STRING> Fun </STRING>
      <STRING> Theorems </STRING>
      <STRING> o_apply </STRING>
    </STRINGLIST>
    <THEOREM>
      <ID> o_apply </ID>
      <MATHML>
        <ISA> (?f o ?g) ?x = ?f (?g ?x) </ISA>
      </MATHML>
    </THEOREM>
    <PROOF>
      <EXTREF>
        <TEXT> Proof of the theorem </TEXT>
        <URL> http://www.ist.tugraz.at/projects/isac/www/kbase/thy/browser_info/HOL/HOL-Real/Isac/Fun.html </URL>
      </EXTREF>
    </PROOF>
    <EXPLANATIONS> </EXPLANATIONS>
    <MATHAUTHORS>
      <STRING> Isabelle team, TU Munich </STRING>
    </MATHAUTHORS>
    <COURSEDESIGNS>
    </COURSEDESIGNS>
  </THEOREMDATA>
  --------------------------------------------------------------------------------
*}
subsubsection {* Changesets of begin and end *}
text {*
  isac development was done between these changesets:
  ------------------------------------------------------------------------------
  Changeset: 42519 (1f3b4270363e) meeting dmeindl: added missing files
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 16:39:30 +0200 (8 months)
  :
  : isac on Isablle2011
  :
  Changeset:41897 (355be7f60389) merged isabisac with Isabelle2011
  Branch: decompose-isar 
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2011-02-25 13:04:56 +0100 (2011-02-25)
  ------------------------------------------------------------------------------
*}

subsection {* isac on Isabelle2009-2 *}
subsubsection {* Summary of development *}
text {*
  In 2009 the update of isac from Isabelle2002 started with switching from CVS to hg.
  The update was painful (bridging 7 years of Isabelle development) and cut short 
  due to the start of introducing Isabelle's contexts (Mathias Lehnfeld) and
  going on to Isabelle2011 although most of the tests did not run.
*}
subsubsection {* Run tests *}
text {*
  WN131021 this is broken by installation of Isabelle2011/12/13,
  because all these write their binaries to ~/.isabelle/heaps/..

  $ cd /usr/local/isabisac09-2/
  $ ./bin/isabelle emacs -l HOL src/Tools/isac/Build_Isac.thy
  $ ./bin/isabelle emacs -l Isac src/Tools/isac/Test_Isac.thy
  NOT THE RIGHT VERSION.....    test/Tools/isac/Test_Isac.thy !!!
*}
subsubsection {* State of tests *}
text {* 
  Most tests are broken by the update from Isabelle2002 to Isabelle2009-2.
*}
subsubsection {* Changesets of begin and end *}
text {*
  isac development was done between these changesets:
  ------------------------------------------------------------------------------
  Changeset: 38115 (940a5feea094) Little improvements of isac-plugin
  Branch: decompose-isar 
  User: Marco Steger <m.steger@student.tugraz.at>
  Date: 2011-02-06 18:30:28 +0100 (2011-02-06)
  :
  : isac on Isablle2009-2
  :
  Changeset: 37870 (5100a9c3abf8) created branch isac-from-Isabelle2009-2
  Branch: isac-from-Isabelle2009-2 
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2010-07-21 09:59:35 +0200 (2010-07-21)
  ------------------------------------------------------------------------------
*}

subsection {* isac on Isabelle2002 *}
subsubsection {* Summary of development *}
text {*
  From 1999 to 2010 all the basic functionality (except "ctxt" and "error pattern")
  of isac's mathematics engine has been implemented.
*}
subsubsection {* Run tests *}
subsubsection {* State of tests *}
text {* 
  All tests work on an old notebook (the right PolyML coudn't be upgraded to more
  recent Linux versions)
*}
subsubsection {* Changesets of begin and end *}
text {*
  Up to the above Mercurial changeset 5100a9c3abf8 isac used CVS;
  see https://intra.ist.tugraz.at/hg/isac containing a conversion to Mercurial.
*}

end
(*========== inhibit exn 130719 Isabelle2013 ===================================
============ inhibit exn 130719 Isabelle2013 =================================*)

(*-.-.-.-.-.-isolate response.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
  -.-.-.-.-.-.-isolate response.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.*)

